require_relative 'smells/attribute'
require_relative 'smells/boolean_parameter'
require_relative 'smells/class_variable'
require_relative 'smells/control_parameter'
require_relative 'smells/data_clump'
require_relative 'smells/duplicate_method_call'
require_relative 'smells/feature_envy'
require_relative 'smells/irresponsible_module'
require_relative 'smells/long_parameter_list'
require_relative 'smells/long_yield_list'
require_relative 'smells/module_initialize'
require_relative 'smells/nested_iterators'
require_relative 'smells/nil_check'
require_relative 'smells/prima_donna_method'
require_relative 'smells/repeated_conditional'
require_relative 'smells/too_many_instance_variables'
require_relative 'smells/too_many_methods'
require_relative 'smells/too_many_statements'
require_relative 'smells/uncommunicative_method_name'
require_relative 'smells/uncommunicative_module_name'
require_relative 'smells/uncommunicative_parameter_name'
require_relative 'smells/uncommunicative_variable_name'
require_relative 'smells/unused_parameters'
require_relative 'smells/unused_private_method'
require_relative 'smells/utility_function'
