require_relative 'reference_collector'

require_relative 'sexp_extensions/arguments'
require_relative 'sexp_extensions/attribute_assignments'
require_relative 'sexp_extensions/block'
require_relative 'sexp_extensions/case'
require_relative 'sexp_extensions/constant'
require_relative 'sexp_extensions/if'
require_relative 'sexp_extensions/literal'
require_relative 'sexp_extensions/logical_operators'
require_relative 'sexp_extensions/methods'
require_relative 'sexp_extensions/module'
require_relative 'sexp_extensions/nested_assignables'
require_relative 'sexp_extensions/send'
require_relative 'sexp_extensions/super'
require_relative 'sexp_extensions/symbols'
require_relative 'sexp_extensions/variables'
require_relative 'sexp_extensions/when'
require_relative 'sexp_extensions/yield'
